#ifndef __ltrapitypes__
#define __ltrapitypes__
#include "lwintypes.h"
#include "ltrapidefine.h"
#ifdef __cplusplus
extern "C" {
#endif
//
#ifndef COMMENT_LENGTH
#define COMMENT_LENGTH 256
#endif
#ifndef ADC_CALIBRATION_NUMBER
#define ADC_CALIBRATION_NUMBER 256
#endif
#ifndef DAC_CALIBRATION_NUMBER
#define DAC_CALIBRATION_NUMBER 256
#endif
//
#pragma pack(4)

typedef struct __ERROR_STRING_DEF__
    {
    INT code;
    LPCSTR message;
    }
    T_ERROR_STRING_DEF;

// Информация о типе крейта
typedef struct _CRATE_INFO_
{
 BYTE CrateType;                                           // Тип крейта
 BYTE CrateInterface;                                      // Тип подключения крейта
} TCRATE_INFO;

// описание модуля
typedef struct _DESCRIPTION_MODULE_                        //
{                                                          //
  BYTE   CompanyName[16];                                  //
  BYTE   DeviceName[16];                                   // название изделия
  BYTE   SerialNumber[16];                                 // серийный номер изделия
  BYTE   Revision;                                         // ревизия изделия
  BYTE   Comment[COMMENT_LENGTH];                          //
} TDESCRIPTION_MODULE;                                     //
// описание процессора и програмного обеспечения
typedef struct _DESCRIPTION_CPU_                           //
{                                                          //
  BYTE   Active;                                           // флаг достоверности остальных полей структуры
  BYTE   Name[16];                                         // название
  double ClockRate;                                        //
  DWORD  FirmwareVersion;                                  //
  BYTE   Comment[COMMENT_LENGTH];                          //
} TDESCRIPTION_CPU;                                        //
// описание плис
typedef struct _DESCRIPTION_FPGA_                          //
{                                                          //
  BYTE   Active;                                           // флаг достоверности остальных полей структуры
  BYTE   Name[16];                                         // название
  double ClockRate;                                        //
  DWORD  FirmwareVersion;                                  //
  BYTE   Comment[COMMENT_LENGTH];                          //
} TDESCRIPTION_FPGA;                                       //
// описание ацп
typedef struct _DESCRIPTION_ADC_                           //
{                                                          //
  BYTE   Active;                                           // флаг достоверности остальных полей структуры
  BYTE   Name[16];                                         // название
  double Calibration[ADC_CALIBRATION_NUMBER];              // корректировочные коэффициенты
  BYTE   Comment[COMMENT_LENGTH];                          //
} TDESCRIPTION_ADC;                                        //
// описание цап
typedef struct _DESCRIPTION_DAC_                           //
{                                                          //
  BYTE   Active;                                           // флаг достоверности остальных полей структуры
  BYTE   Name[16];                                         // название
  double Calibration[DAC_CALIBRATION_NUMBER];              // корректировочные коэффициенты
  BYTE   Comment[COMMENT_LENGTH];                          //
} TDESCRIPTION_DAC;                                        //
// описание h-мезанинов
typedef struct _DESCRIPTION_MEZZANINE_                     //
{                                                          //
  BYTE   Active;                                           // флаг достоверности остальных полей структуры
  BYTE   Name[16];                                         // название изделия
  BYTE   SerialNumber[16];                                 // серийный номер изделия
  BYTE   Revision;                                         // ревизия изделия
  double Calibration[4];                                   // корректировочные коэффициенты
  BYTE   Comment[COMMENT_LENGTH];                          // комментарий
} TDESCRIPTION_MEZZANINE;                                   //
// описание цифрового вв
typedef struct _DESCRIPTION_DIGITAL_IO_                    //
{                                                          //
  BYTE   Active;                                           // флаг достоверности остальных полей структуры
  BYTE   Name[16];                                         // название ???????
  WORD   InChannels;                                       // число каналов
  WORD   OutChannels;                                      // число каналов
  BYTE   Comment[COMMENT_LENGTH];                          //
} TDESCRIPTION_DIGITAL_IO;                                 //
// описание интерфейсных модулей
typedef struct _DESCRIPTION_INTERFACE_                     //
{                                                          //
  BYTE   Active;                                           // флаг достоверности остальных полей структуры
  BYTE   Name[16];                                         // название
  BYTE   Comment[COMMENT_LENGTH];                          //
} TDESCRIPTION_INTERFACE;                                  //

// элемент списка IP-крейтов
typedef struct _IPCRATE_ENTRY_
    {
    DWORD ip_addr;                                          // IP адрес (host-endian)
    DWORD flags;                                            // флаги режимов (CRATE_IP_FLAG_...)
    CHAR serial_number[16];                                 // серийный номер (если крейт подключен)
    BYTE is_dynamic;                                        // 0 = задан пользователем, 1 = найден автоматически
    BYTE status;                                            // состояние (CRATE_IP_STATUS_...)
    }
    TIPCRATE_ENTRY;

typedef struct
{
    DWORD size; /** размер струкуры */
    DWORD flags; /** флаги */
    WORD crate_type;
    WORD crate_intf;
    WORD crate_state;
    WORD crate_mode;
    QWORD con_time;
    WORD res[11];
    WORD modules_cnt;
    WORD mids[CRATE_MAX];
    WORD res2[3*CRATE_MAX];
    WORD ctl_clients_cnt; /** количество клиентов, подключенных по управляющему каналу к крейту */
    WORD total_mod_clients_cnt; /** количество клиентов, подключенные ко всем модулям */
    DWORD res3[11];

    QWORD  wrd_sent; /** передано слов крейту */
    QWORD  wrd_recv; /** слов принято от крейта и его модулей */
    double bw_send; /** скорость передачи слов */
    double bw_recv; /** скорость приема слов */
    QWORD  crate_wrd_recv; /** количество управляющих слов непосредственно от крейта */
    QWORD  internal_rbuf_miss; /** количество потерянных буферов из-за внутреннего переполнения */
    DWORD  internal_rbuf_ovfls; /** количество переполнений внутреннго буфера крейта */
    DWORD  rbuf_ovfls; /** количество переполнений буфера для модулей крейта */
    DWORD  total_start_marks; /** кол-во меток старта как от крейта, так и от модулей */
    DWORD  total_sec_marks; /** кол-во секундных меток как от крейта, так и от модулей */
    DWORD  crate_start_marks; /** кол-во меток старта непосредственно от крейта */
    DWORD  crate_sec_marks; /** кол-во секундных меток непосредственно от крейта */
    DWORD  res4[30];
} TLTR_CRATE_STATISTIC;

typedef struct
{
    DWORD size; /** размер структуры */
    WORD client_cnt; /** количество клиентов */
    WORD mid;
    DWORD flags; /** флаги */
    DWORD res[9];
    QWORD wrd_sent; /** кол-во переданных слов модулю */
    QWORD wrd_recv; /** кол-во принятых слов от модуля */
    double bw_send; /** скорость передачи слов */
    double bw_recv; /** скорость приема слов */
    QWORD wrd_sent_to_client; /** сколько слов передано клиенту */
    QWORD wrd_recv_from_client; /** сколько слов принято от клиентов */    
    QWORD wrd_recv_drop; /** сколько выкинуто слов из за переполнения */
    DWORD rbuf_ovfls; /** количество переполнений буфера на прием */
    DWORD send_srvbuf_size; /** размер буфера для модуля на передачу */
    DWORD recv_srvbuf_size; /** размер буфера для модуля на прием */
    DWORD send_srvbuf_full; /** на сколько слов заполнен буфер на передачу */
    DWORD recv_srvbuf_full; /** на сколько слов заполнен буфер на прием */
    DWORD send_srvbuf_full_max; /** на сколько слов был максимально заполнен буфер на передачу */
    DWORD recv_srvbuf_full_max; /** на сколько слов был максимально заполнен буфер на прием */
    DWORD res2[17];    /** резерв */
    DWORD start_mark; /** количество меток старт от модуля */
    DWORD sec_mark; /** количество секундных меток от модуля */
    /* Поля, описывающие состояние внутренней аппаратной очереди модуля.
     * Действительны, только если есть флаг LTR_MODULE_FLAGS_USE_HARD_SEND_FIFO */
    DWORD hard_send_fifo_size; /**< Размер аппаратной очереди */
    DWORD hard_send_fifo_unack_words; /**< Заполненность аппаратной очереди (количество отправленных, но не подтвержденных слов)*/
    DWORD hard_send_fifo_underrun; /**< Количество голоданий очереди после последнего сброса */
    DWORD hard_send_fifo_overrun; /**< Количество переполнений очереди после последнего сброса */
    DWORD hard_send_fifo_internal; /** Внутренне состояние аппаратной очереди */

    DWORD res3[25];    /** резерв */
} TLTR_MODULE_STATISTIC;


/** информация о устройстве и прошивке  */
typedef struct
{
    DWORD size;
    char devname[LTR_CRATE_DEVNAME_SIZE]; /**< название устройства */
    char serial[LTR_CRATE_SERIAL_SIZE];   /**< серийный номер */
    char soft_ver[LTR_CRATE_SOFTVER_SIZE]; /**< версия прошивки */
    char brd_revision[LTR_CRATE_REVISION_SIZE]; /**< ревизия платы */
    char brd_impl[LTR_CRATE_IMPLEMENTATION_SIZE]; /**< опции платы */
    char bootloader_ver[LTR_CRATE_BOOTVER_SIZE]; /**< версия загрузчика */
    char cpu_type[LTR_CRATE_CPUTYPE_SIZE]; /**< тип микроконтроллера */
    char fpga_name[LTR_CRATE_FPGA_NAME_SIZE];
    char fpga_version[LTR_CRATE_FPGA_VERSION_SIZE];
    char spec_info[LTR_CRATE_SPECINFO_SIZE]; /**< резерв */
} TLTR_CRATE_DESCR;

#pragma pack()
//
#ifdef __cplusplus
}
#endif
#endif
